//______________________________________________________________________________
//   |       |
//   | knipp |           Knipp Medien und Kommunikation GmbH
//    -------
//
//
//     Knipp RRI Toolkit
//
//
//     Copyright (C) 2005 by Knipp Medien und Kommunikation GmbH
//
//     This library is free software; you can redistribute it and/or
//     modify it under the terms of the GNU Lesser General Public License 
//     as published by the Free Software Foundation; either version 2.1 
//     of the License, or any later version.
// 
//     This library is distributed in the hope that it will be useful, 
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
//     GNU Lesser General Public License for more details.
//
//     You should have received a copy of the GNU Lesser General
//     Public License along with this library; if not, write to the 
//     Free Software Foundation, Inc., 59 Temple Place, Suite 330, 
//     Boston, MA 02111-1307 USA 
//
//
//     initial author:    Klaus Malorny
//
//     version:           1.7 (1007), 2019/05/08 12:15:02
//                        $Revision: 937 $ 
//                        $Date: 2017-10-09 17:29:32 +0200 (Mo, 09 Okt 2017) $
//                        $Author: Klaus.Malorny $
//

package de.knipp.rritoolkit;

import java.io.IOException;


/**
 *  interface to a class that can produce new channels to the registry.
 *  An implementation must be thread-safe.
 *
 *  @version  $Revision: 937 $
 *  @author   Klaus Malorny
 */

public interface ChannelFactory
{
  /**
   *  create a new channel. If a implementation or configuration specific
   *  limit of number of channels exists and this limit has been reached,
   *  {@code null} is returned. If a channel could not be created for
   *  a different reason, an {@code IOException} is thrown. Channels that
   *  are no longer used by the caller should be closed
   *
   *  @return                    the channel or {@code null} if no more
   *                             channels are available
   *  @throws IOException        if the channel could not be created
   */
  
  public RegistryChannel createChannel ()
    throws IOException;
  
 
}



