//______________________________________________________________________________
//   |       |
//   | knipp |           Knipp Medien und Kommunikation GmbH
//    -------
//
//
//     Knipp RRI Toolkit
//
//
//     Copyright (C) 2005 by Knipp Medien und Kommunikation GmbH
//
//     This library is free software; you can redistribute it and/or
//     modify it under the terms of the GNU Lesser General Public License 
//     as published by the Free Software Foundation; either version 2.1 
//     of the License, or any later version.
// 
//     This library is distributed in the hope that it will be useful, 
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
//     GNU Lesser General Public License for more details.
//
//     You should have received a copy of the GNU Lesser General
//     Public License along with this library; if not, write to the 
//     Free Software Foundation, Inc., 59 Temple Place, Suite 330, 
//     Boston, MA 02111-1307 USA 
//
//
//     initial author:    Klaus Malorny
//
//     version:           1.7 (1007), 2019/05/08 12:15:02
//                        $Revision: 937 $ 
//                        $Date: 2017-10-09 17:29:32 +0200 (Mo, 09 Okt 2017) $
//                        $Author: Klaus.Malorny $
//

package de.knipp.rritoolkit.dns;

import org.w3c.dom.Element;
import org.w3c.dom.DOMException;

import de.knipp.rritoolkit.ResponseParseException;




/**
 *  class that describes an "A" record in the domain.
 *
 *  @version  $Revision: 937 $
 *  @author   Klaus Malorny
 */

public class DnsEntryA extends DnsEntryAddress
{

  
  /**
   *  constructor
   *
   *  @param owner                       the owner
   *  @param address                     the address
   */
  
  public DnsEntryA (String owner, String address)
  {
    super (owner, address);
  }
  

  
  /**
   *  constructor
   *
   *  @param el                       the element
   *  @throws DOMException            on DOM exceptions
   *  @throws ResponseParseException  on wrong content
   */
  
  protected DnsEntryA (Element el)
    throws DOMException, ResponseParseException
  {
    super (el);
  }
  


  /**
   *  accept the given visitor
   *
   *  @param visitor                  the visitor
   */

  @Override
  public void accept (DnsEntryVisitor visitor)
  {
    visitor.visit (this);
  }
  
  

 
  /**
   *  create a clone of this instance.
   *
   *  @return                         the clone
   */

  @Override
  public DnsEntryA clone ()
  {
    return (DnsEntryA) super.clone ();
  }
  
  
  /**
   *  return the record name
   *
   *  @return               the record name
   */

  @Override
  protected String getRecordName ()
  {
    return "A";
  }


  /**
   *  return the (local) element name
   *
   *  @return               the element name
   */

  @Override
  protected String getElementName ()
  {
    return "A";
  }

}

