//______________________________________________________________________________
//   |       |
//   | knipp |           Knipp Medien und Kommunikation GmbH
//    -------
//
//
//     Knipp RRI Toolkit
//
//
//     Copyright (C) 2005 by Knipp Medien und Kommunikation GmbH
//
//     This library is free software; you can redistribute it and/or
//     modify it under the terms of the GNU Lesser General Public License 
//     as published by the Free Software Foundation; either version 2.1 
//     of the License, or any later version.
// 
//     This library is distributed in the hope that it will be useful, 
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
//     GNU Lesser General Public License for more details.
//
//     You should have received a copy of the GNU Lesser General
//     Public License along with this library; if not, write to the 
//     Free Software Foundation, Inc., 59 Temple Place, Suite 330, 
//     Boston, MA 02111-1307 USA 
//
//
//     initial author:    Klaus Malorny
//
//     version:           1.7 (1007), 2019/05/08 12:15:02
//                        $Revision: 950 $ 
//                        $Date: 2018-01-30 18:36:44 +0100 (Di, 30 Jan 2018) $
//                        $Author: klaus.malorny $
//

package de.knipp.rritoolkit.message;


/**
 *  interface for a functional visitor of a {@code Message}
 * 
 *  @param <R>                the return value type
 *
 *  @version  $Revision: 950 $
 *  @author   Klaus Malorny
 */

public interface MessageFuncVisitor<R>
{
  /**
   *  called by a ChangeProvAuthInfo message
   *
   *  @param msg            the message
   *  @return               the return value
   */

  public R visit (ChangeProvAuthInfoMessage msg);
  

  
  /**
   *  called by a DomainExpire message
   *
   *  @param msg            the message
   *  @return               the return value
   */
  
  public R visit (DomainExpireMessage msg);
  
  
  
  /**
   *  called by a DomainExpireWarning message
   *
   *  @param msg            the message
   *  @return               the return value
   */
  
  public R visit (DomainExpireWarningMessage msg);

  

  /**
   *  called by an AuthInfoExpire message
   *
   *  @param msg            the message
   *  @return               the return value
   */

  public R visit (AuthInfoExpireMessage msg);



  /**
   *  called by an AuthInfo2Notify message
   *
   *  @param msg            the message
   *  @return               the return value
   */

  public R visit (AuthInfo2NotifyMessage msg);



  /**
   *  called by a DomainDelete message
   *
   *  @param msg            the message
   *  @return               the return value
   */

  public R visit (DomainDeleteMessage msg);


  /**
   *  called by an AuthInfo2Delete message
   *
   *  @param msg            the message
   *  @return               the return value
   */

  public R visit (AuthInfo2DeleteMessage msg);

}


