//______________________________________________________________________________
//   |       |
//   | knipp |           Knipp Medien und Kommunikation GmbH
//    -------
//
//
//     Knipp RRI Toolkit
//
//
//     Copyright (C) 2005 by Knipp Medien und Kommunikation GmbH
//
//     This library is free software; you can redistribute it and/or
//     modify it under the terms of the GNU Lesser General Public License 
//     as published by the Free Software Foundation; either version 2.1 
//     of the License, or any later version.
// 
//     This library is distributed in the hope that it will be useful, 
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
//     GNU Lesser General Public License for more details.
//
//     You should have received a copy of the GNU Lesser General
//     Public License along with this library; if not, write to the 
//     Free Software Foundation, Inc., 59 Temple Place, Suite 330, 
//     Boston, MA 02111-1307 USA 
//
//
//     initial author:    Klaus Malorny
//
//     version:           1.7 (1007), 2019/05/08 12:15:02
//                        $Revision: 998 $ 
//                        $Date: 2019-04-18 15:17:08 +0200 (Thu, 18 Apr 2019) $
//                        $Author: klaus.malorny $
//

package de.knipp.rritoolkit.request;

import java.io.IOException;

import org.w3c.dom.Node;
import org.w3c.dom.DOMException;

import de.knipp.rritoolkit.Port;
import de.knipp.rritoolkit.KVList;
import de.knipp.rritoolkit.XmlBuilder;
import de.knipp.rritoolkit.EncodeException;

import de.knipp.rritoolkit.response.ContactInfoResponse;

import static de.knipp.rritoolkit.DenicConstants.NS_CONTACT;
import static de.knipp.rritoolkit.DenicConstants.PREFIX_CONTACT;




/**
 *  request to retrieve information about a given contact handle
 *
 *  @version  $Revision: 998 $
 *  @author   Klaus Malorny
 */

public class ContactInfoRequest extends Request
{
  /** the handle */
  
  private String handle;
  
  
  /**
   *  constructor
   */
  
  public ContactInfoRequest ()
  {
    // empty
  }
  
  
  /**
   *  constructor
   *
   *  @param theHandle               the handle
   */
  
  public ContactInfoRequest (String theHandle)
  {
    setHandle (theHandle);
  }

  
  
  //--- implementation of Request ----------------------------------------------
  
  
  /**
   *  execute the this request on the given port. 
   *
   *  @param port                 the port on which this request shall
   *                              be executed
   *  @return                     the response
   *  @throws IOException         if the execution failed
   */

  @Override
  public ContactInfoResponse execute (Port port)
    throws IOException
  {
    return (ContactInfoResponse) super.execute (port);
  }
  

    
  /**
   *  method that needs to be implemented by subclasses for the XML conversion
   *
   *  @param node                 the node where the request shall be added to
   *  @throws DOMException        on DOM exceptions
   *  @throws EncodeException     if the encoding fails for some reason
   */

  @Override
  protected void requestToXml (Node node)
    throws DOMException, EncodeException
  {
    if (handle == null) 
      throw new EncodeException ("handle not set");
    
    XmlBuilder xb = new XmlBuilder (node);

    xb.
      es (NS_CONTACT, PREFIX_CONTACT, "info").
      elt (NS_CONTACT, PREFIX_CONTACT, "handle", handle).
      ee ();
  }
  

  
  
  
  /**
   *  return the action string for this request
   *
   *  @return                     the action string
   */

  @Override
  protected String getActionCode ()
  {
    return "info";
  }
  

  
  /**
   *  method that needs to be implemented by subclasses for the 
   *  key-value list conversion
   *
   *  @param list                   the list that has to be filled
   *  @throws EncodeException     if the encoding fails for some reason
   */

  @Override
  protected void requestToKVList (KVList list)
    throws EncodeException
  {
    if (handle == null)
      throw new EncodeException ("handle not set");
    
    list.set ("handle", handle);
  }
  
  
  
  
  /**
   *  return a string representation for debugging purposes
   *
   *  @return                 the representation
   */

  @Override
  public String toString ()
  {
    return super.toString () + "  handle: " + handle + "\n";
  }
  
  
  //--- access methods ---------------------------------------------------------
  
  /**
   *  return the handle
   *
   *  @return                      the handle
   */
  
  public String getHandle ()
  {
    return handle;
  }
  
  
  
  /**
   *  set the handle
   *
   *  @param theHandle             the handle
   */
  
  public void setHandle (String theHandle)
  {
    handle = theHandle;
  }
  
  
  
}
