//______________________________________________________________________________
//   |       |
//   | knipp |           Knipp Medien und Kommunikation GmbH
//    -------
//
//
//     Knipp RRI Toolkit
//
//
//     Copyright (C) 2005 by Knipp Medien und Kommunikation GmbH
//
//     This library is free software; you can redistribute it and/or
//     modify it under the terms of the GNU Lesser General Public License 
//     as published by the Free Software Foundation; either version 2.1 
//     of the License, or any later version.
// 
//     This library is distributed in the hope that it will be useful, 
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
//     GNU Lesser General Public License for more details.
//
//     You should have received a copy of the GNU Lesser General
//     Public License along with this library; if not, write to the 
//     Free Software Foundation, Inc., 59 Temple Place, Suite 330, 
//     Boston, MA 02111-1307 USA 
//
//
//     initial author:    Klaus Malorny
//
//     version:           1.7 (1007), 2019/05/08 12:15:02
//                        $Revision: 937 $ 
//                        $Date: 2017-10-09 17:29:32 +0200 (Mo, 09 Okt 2017) $
//                        $Author: Klaus.Malorny $
//

package de.knipp.rritoolkit.request;

import java.io.IOException;

import de.knipp.rritoolkit.Port;

import de.knipp.rritoolkit.response.DomainCheckResponse;


/**
 *  request to retrieve information about a given domain
 *
 *  @version  $Revision: 937 $
 *  @author   Klaus Malorny
 */

public class DomainCheckRequest extends DomainSimpleRequest
{
  
  
  /**
   *  constructor
   */
  
  public DomainCheckRequest ()
  {
    // empty
  }
  
  

  /**
   *  constructor
   *
   *  @param theDomainName          the domain name
   *  @param theEncodedDomainName   the encoded domain name, may be {@code null}
   *                                for ASCII-only domains
   */
  
  public DomainCheckRequest (String theDomainName, String theEncodedDomainName)
  {
    super (theDomainName, theEncodedDomainName);
  }


  
  /**
   *  constructor
   *
   *  @param theDomainName             the domain name
   */
  
  public DomainCheckRequest (String theDomainName)
  {
    super (theDomainName);
  }
  
  
  //--- implementation of Request ----------------------------------------------
  
  /**
   *  execute the this request on the given port. 
   *
   *  @param port                 the port on which this request shall
   *                              be executed
   *  @return                     the response
   *  @throws IOException         if the execution failed
   */
  
  @Override
  public DomainCheckResponse execute (Port port)
    throws IOException
  {
    return (DomainCheckResponse) super.execute (port);
  }
  
    
    
  /**
   *  return the XML element name
   *
   *  @return                      the element name
   */

  @Override
  protected String getRequestElementName ()
  {
    return "check";
  }
  
  
  
  /**
   *  return the action string for this request
   *
   *  @return                     the action string
   */

  @Override
  protected String getActionCode ()
  {
    return "check";
  }
  
  
  
  
  
}


