//______________________________________________________________________________
//   |       |
//   | knipp |           Knipp Medien und Kommunikation GmbH
//    -------
//
//
//     Knipp RRI Toolkit
//
//
//     Copyright (C) 2005 by Knipp Medien und Kommunikation GmbH
//
//     This library is free software; you can redistribute it and/or
//     modify it under the terms of the GNU Lesser General Public License 
//     as published by the Free Software Foundation; either version 2.1 
//     of the License, or any later version.
// 
//     This library is distributed in the hope that it will be useful, 
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
//     GNU Lesser General Public License for more details.
//
//     You should have received a copy of the GNU Lesser General
//     Public License along with this library; if not, write to the 
//     Free Software Foundation, Inc., 59 Temple Place, Suite 330, 
//     Boston, MA 02111-1307 USA 
//
//
//     initial author:    Klaus Malorny
//
//     version:           1.7 (1007), 2019/05/08 12:15:02
//                        $Revision: 937 $ 
//                        $Date: 2017-10-09 17:29:32 +0200 (Mo, 09 Okt 2017) $
//                        $Author: Klaus.Malorny $
//

package de.knipp.rritoolkit.request;

import java.io.IOException;

import org.w3c.dom.Node;
import org.w3c.dom.DOMException;

import de.knipp.rritoolkit.Port;
import de.knipp.rritoolkit.KVList;
import de.knipp.rritoolkit.XmlBuilder;
import de.knipp.rritoolkit.EncodeException;

import de.knipp.rritoolkit.response.LoginResponse;

import static de.knipp.rritoolkit.DenicConstants.NS_GLOBAL;

/**
 *  implementation of the login request
 *
 *  @version  $Revision: 937 $
 *  @author   Klaus Malorny
 */

public class LoginRequest extends Request
{
  /** the user name */
  
  private String userName;
  
  
  /** the password */
  
  private String password;
  
  

  /**
   *  set the user name
   *
   *  @param theUserName             the user name
   */
  
  public void setUser (String theUserName)
  {
    userName = theUserName;
  }
  
  
  /**
   *  return the user name
   *
   *  @return                        the user name
   */
  
  public String getUser ()
  {
    return userName;
  }
  
  
  /**
   *  set the password
   *
   *  @param thePassword              the password
   */
  
  public void setPassword (String thePassword)
  {
    password = thePassword;
  }
  
  
  
  /**
   *  return the password
   *
   *  @return                         the password
   */
  
  public String getPassword ()
  {
    return password;
  }
  
  
  
  //--- implementation of Request ----------------------------------------------
  

  /**
   *  execute the this request on the given port. 
   *
   *  @param port                 the port on which this request shall
   *                              be executed
   *  @return                     the response
   *  @throws IOException         if the execution failed
   */
  
  @Override
  public LoginResponse execute (Port port)
    throws IOException
  {
    return (LoginResponse) super.execute (port);
  }

  
  
  /**
   *  method that needs to be implemented by subclasses for the XML conversion
   *
   *  @param node                 the node where the request shall be added to
   *  @throws DOMException        on DOM exceptions
   *  @throws EncodeException     if the encoding fails for some reason
   */

  @Override
  protected void requestToXml (Node node)
    throws DOMException, EncodeException
  {
    XmlBuilder xb = new XmlBuilder (node);
    
    xb.
      ns (NS_GLOBAL).
      es ("login").
      elt ("user", userName).
      elt ("password", password).
      ee ();
  }
  
  
  
  
  /**
   *  return the action string for this request
   *
   *  @return                     the action string
   */

  @Override
  protected String getActionCode ()
  {
    return "login";
  }
  

  
  /**
   *  method that needs to be implemented by subclasses for the 
   *  key-value list conversion
   *
   *  @param list                   the list that has to be filled
   *  @throws EncodeException       if the encoding fails for some reason
   */

  @Override
  protected void requestToKVList (KVList list)
    throws EncodeException
  {
    list.set ("user", userName);
    list.set ("password", password);
  }
  

 
  /**
   *  return a string representation for debugging purposes
   *
   *  @return                      the string representation
   */
  
  @Override
  public String toString ()
  {
    return super.toString () + "  user name: " + userName + "\n" + 
      "  password: " + password + "\n";
  }

}



