//______________________________________________________________________________
//   |       |
//   | knipp |           Knipp Medien und Kommunikation GmbH
//    -------
//
//
//     Knipp RRI Toolkit
//
//
//     Copyright (C) 2005 by Knipp Medien und Kommunikation GmbH
//
//     This library is free software; you can redistribute it and/or
//     modify it under the terms of the GNU Lesser General Public License 
//     as published by the Free Software Foundation; either version 2.1 
//     of the License, or any later version.
// 
//     This library is distributed in the hope that it will be useful, 
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
//     GNU Lesser General Public License for more details.
//
//     You should have received a copy of the GNU Lesser General
//     Public License along with this library; if not, write to the 
//     Free Software Foundation, Inc., 59 Temple Place, Suite 330, 
//     Boston, MA 02111-1307 USA 
//
//
//     initial author:    Klaus Malorny
//
//     version:           1.7 (1007), 2019/05/08 12:15:02
//                        $Revision: 937 $ 
//                        $Date: 2017-10-09 17:29:32 +0200 (Mo, 09 Okt 2017) $
//                        $Author: Klaus.Malorny $
//

package de.knipp.rritoolkit.response;



/**
 *  interface for a visitor of a {@code DenicResponse}
 *
 *  @version  $Revision: 937 $
 *  @author   Klaus Malorny
 */

public interface ResponseVisitor
{
  /**
   *  called by a generic response
   *
   *  @param response                 the response
   */
  
  public void visit (Response response);
  
  
  /**
   *  called by a contact check response
   *
   *  @param response                 the response
   */
  
  public void visit (ContactCheckResponse response);
  
  
  
  /**
   *  calle by a contact create response
   *
   *  @param response                 the response
   */
  
  public void visit (ContactCreateResponse response);
  
  
  
  /**
   *  called by a contact info response
   *
   *  @param response                 the response
   */
  
  public void visit (ContactInfoResponse response);

  
  
  /**
   *  called by a contact update response
   *
   *  @param response                 the response
   */
  
  public void visit (ContactUpdateResponse response);
  
  
  
  /**
   *  called by a domain change holder response
   *
   *  @param response                 the response
   */
  
  public void visit (DomainChangeHolderResponse response);
  
  
  /**
   *  called by a domain change prov response
   *
   *  @param response                 the response
   */
  
  public void visit (DomainChangeProvResponse response);
  
  
  /**
   *  called by a domain check response
   *
   *  @param response                 the response
   */
  
  public void visit (DomainCheckResponse response);
  
  
  
  /**
   *  called by a domain create response
   *
   *  @param response                 the response
   */
  
  public void visit (DomainCreateResponse response);
  

  
  /**
   *  called by a domain delete response
   *
   *  @param response                 the response
   */
  
  public void visit (DomainDeleteResponse response);
  
  
  /**
   *  called by a domain restore response
   *
   *  @param response                 the response
   */
  
  public void visit (DomainRestoreResponse response);
  
  
  /**
   *  called by a domain info response
   *
   *  @param response                 the response
   */
  
  public void visit (DomainInfoResponse response);
  
  
  
  /**
   *  called by a domain transit response
   *
   *  @param response                 the response
   */
  
  public void visit (DomainTransitResponse response);
  
  
  /**
   *  called by a domain update response
   *
   *  @param response                 the response
   */
  
  public void visit (DomainUpdateResponse response);
  

  /**
   *  called by a login response
   *
   *  @param response                 the response
   */
  
  public void visit (LoginResponse response);
  
  
  /**
   *  called by a logout response
   *
   *  @param response                 the response
   */
  
  public void visit (LogoutResponse response);
  
  
  /**
   *  called by queue delete response
   *
   *  @param response                 the response
   */
  
  public void visit (QueueDeleteResponse response);
  
  
  /**
   *  called by a queue read response
   *
   *  @param response                 the response
   */
  
  public void visit (QueueReadResponse response);


  /**
   *  called by an AuthInfo 1 create response
   *
   *  @param response                 the response
   */

  public void visit (AuthInfo1CreateResponse response);


  /**
   *  called by an AuthInfo 1 delete response
   *
   *  @param response                 the response
   */

  public void visit (AuthInfo1DeleteResponse response);


  /**
   *  called by an AuthInfo 2 create response
   *
   *  @param response                 the response
   */

  public void visit (AuthInfo2CreateResponse response);

}


