//______________________________________________________________________________
//   |       |
//   | knipp |           Knipp Medien und Kommunikation GmbH
//    -------
//
//
//     Knipp RRI Toolkit
//
//
//     Copyright (C) 2005 by Knipp Medien und Kommunikation GmbH
//
//     This library is free software; you can redistribute it and/or
//     modify it under the terms of the GNU Lesser General Public License 
//     as published by the Free Software Foundation; either version 2.1 
//     of the License, or any later version.
// 
//     This library is distributed in the hope that it will be useful, 
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
//     GNU Lesser General Public License for more details.
//
//     You should have received a copy of the GNU Lesser General
//     Public License along with this library; if not, write to the 
//     Free Software Foundation, Inc., 59 Temple Place, Suite 330, 
//     Boston, MA 02111-1307 USA 
//
//
//     initial author:    Klaus Malorny
//
//     version:           1.7 (1007), 2019/05/08 12:15:02
//                        $Revision: 1001 $ 
//                        $Date: 2019-04-23 16:56:01 +0200 (Tue, 23 Apr 2019) $
//                        $Author: klaus.malorny $
//

package de.knipp.rritoolkit.response;
  



/**
 *  a simple pattern, consisting of a single code value
 *
 *  @version  $Revision: 1001 $
 *  @author   Klaus Malorny
 */

public class SimplePattern implements ErrorPattern
{
  /** the code to match */

  private String codeNumber;


  /**
   *  constructor
   *
   *  @param theCode                the code to match
   */

  public SimplePattern (String theCode)
  {
    codeNumber = theCode;
  }


  /**
   *  return the code that matches this pattern
   *
   *  @return                      the pattern
   */
  
  public String getCode ()
  {
    return codeNumber;
  }
  
  
  

  /**
   *  test whether the given error code matches the pattern
   *
   *  @param code                the code to test, may be {@code null}
   *  @return                    {@code true} if it matches
   */

  @Override
  public boolean matches (ErrorCode code)
  {
    return codeNumber.equals (code.getCode ());
  }

  
  
  
  /**
   *  accept a visitor
   *
   *  @param visitor             the visitor
   */

  @Override
  public void accept (ErrorPatternVisitor visitor)
  {
    visitor.visit (this);
  }
  
}    

