//______________________________________________________________________________
//   |       |
//   | knipp |           Knipp Medien und Kommunikation GmbH
//    -------
//
//
//     Knipp RRI Toolkit
//
//
//     Copyright (C) 2005 by Knipp Medien und Kommunikation GmbH
//
//     This library is free software; you can redistribute it and/or
//     modify it under the terms of the GNU Lesser General Public License 
//     as published by the Free Software Foundation; either version 2.1 
//     of the License, or any later version.
// 
//     This library is distributed in the hope that it will be useful, 
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
//     GNU Lesser General Public License for more details.
//
//     You should have received a copy of the GNU Lesser General
//     Public License along with this library; if not, write to the 
//     Free Software Foundation, Inc., 59 Temple Place, Suite 330, 
//     Boston, MA 02111-1307 USA 
//
//
//     initial author:    Klaus Malorny
//
//     version:           1.7 (1007), 2019/05/08 12:15:02
//                        $Revision: 1000 $ 
//                        $Date: 2019-04-23 16:14:31 +0200 (Tue, 23 Apr 2019) $
//                        $Author: klaus.malorny $
//

package de.knipp.rritoolkit.test;

import java.io.FileReader;
import java.io.PrintWriter;
import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.io.IOException;

import java.net.IDN;

import java.util.Map;
import java.util.List;
import java.util.Date;
import java.util.Queue;
import java.util.Optional;
import java.util.ArrayList;
import java.util.Properties;
import java.util.Collections;
import java.util.LinkedHashMap;

import java.util.function.BiFunction;
import java.util.function.Consumer;
import java.util.function.Function;

import java.util.concurrent.ConcurrentLinkedQueue;

import java.util.regex.Pattern;
import java.util.regex.Matcher;

import java.time.Instant;
import java.time.Duration;

import java.time.format.DateTimeParseException;

import de.knipp.rritoolkit.Port;
import de.knipp.rritoolkit.Codec;
import de.knipp.rritoolkit.PooledPort;
import de.knipp.rritoolkit.DomainData;
import de.knipp.rritoolkit.DomainData.Role;
import de.knipp.rritoolkit.ContactData;
import de.knipp.rritoolkit.ChannelFactory;
import de.knipp.rritoolkit.DefaultResponseFactory;

import de.knipp.rritoolkit.dns.DnsEntry;
import de.knipp.rritoolkit.dns.DnsEntryA;
import de.knipp.rritoolkit.dns.DnsEntryMX;
import de.knipp.rritoolkit.dns.DnsEntryNS;
import de.knipp.rritoolkit.dns.DnsEntryAAAA;
import de.knipp.rritoolkit.dns.DnsEntryDNSKEY;

import de.knipp.rritoolkit.message.Message;

import de.knipp.rritoolkit.request.Request;
import de.knipp.rritoolkit.request.QueueReadRequest;
import de.knipp.rritoolkit.request.DomainInfoRequest;
import de.knipp.rritoolkit.request.ContactInfoRequest;
import de.knipp.rritoolkit.request.DomainCheckRequest;
import de.knipp.rritoolkit.request.QueueDeleteRequest;
import de.knipp.rritoolkit.request.DomainUpdateRequest;
import de.knipp.rritoolkit.request.DomainCreateRequest;
import de.knipp.rritoolkit.request.DomainDeleteRequest;
import de.knipp.rritoolkit.request.ContactCheckRequest;
import de.knipp.rritoolkit.request.DomainRestoreRequest;
import de.knipp.rritoolkit.request.DomainTransitRequest;
import de.knipp.rritoolkit.request.ContactUpdateRequest;
import de.knipp.rritoolkit.request.ContactCreateRequest;
import de.knipp.rritoolkit.request.AuthInfo1CreateRequest;
import de.knipp.rritoolkit.request.AuthInfo1DeleteRequest;
import de.knipp.rritoolkit.request.AuthInfo2CreateRequest;
import de.knipp.rritoolkit.request.DomainChangeProvRequest;
import de.knipp.rritoolkit.request.DomainChangeHolderRequest;

import de.knipp.rritoolkit.response.Response;
import de.knipp.rritoolkit.response.QueueReadResponse;
import de.knipp.rritoolkit.response.DomainInfoResponse;

import de.knipp.rritoolkit.tcp.TcpParams;
import de.knipp.rritoolkit.tcp.TcpChannelFactory;



/**
 *  small test program
 *
 *  @version  $Revision: 1000 $
 *  @author   Klaus Malorny
 */

public class Test
{
  /** the buffered reader while parsing */
  
  private BufferedReader batchReader;
  
  
  /** the last parsed arguments during parsing */
  
  private String[] parsedArgs;
  
  
  
  /** the operations */
  
  private static final Map<String, Operation> operations;
  
  
  
  /** pattern to split lines */

  private static Pattern argPattern = Pattern.compile (
    "\\s*(?:\"([^\"]*)\"|(\\S+))\\s*");
  
  
  
  /**
   *  class initializer
   */
  
  static
  {
    operations = new LinkedHashMap<> ();  
      // keep insertion order for proper help output
      
    addOperation (new ContactCU ("contact-create", ContactCreateRequest::new));
    addOperation (new ContactCU ("contact-update", ContactUpdateRequest::new));
    addOperation (new ContactCI ("contact-check", ContactCheckRequest::new));
    addOperation (new ContactCI ("contact-info", ContactInfoRequest::new));
    
    addOperation (new DomainCU ("domain-create", DomainCreateRequest::new,
      false));
    
    addOperation (new DomainCU ("domain-update", DomainUpdateRequest::new,
      false));
    
    addOperation (new DomainCU ("domain-chholder", 
      DomainChangeHolderRequest::new, false));
    
    addOperation (new DomainCU ("domain-chprov", DomainChangeProvRequest::new,
      true));
    
    addOperation (new DomainSimple<> ("domain-transit", 
      DomainTransitRequest::new));
    
    addOperation (new DomainDelete ());

    addOperation (new DomainSimple<> ("domain-restore", 
      DomainRestoreRequest::new));
    
    addOperation (new DomainSimple<> ("domain-check", DomainCheckRequest::new));
    addOperation (new DomainInfo ());
    addOperation (new AuthInfo1Create ());
    
    addOperation (new DomainSimple<> ("authinfo1-delete",
      AuthInfo1DeleteRequest::new));
    
    addOperation (new DomainSimple<> ("authinfo2-create",
      AuthInfo2CreateRequest::new));
    
    addOperation (new QueueRead ());
    addOperation (new QueueDelete ());
    addOperation (new QueueDeleteAll ());
    
  }
  
  
  /**
   *  add an operation
   * 
   *  @param op             the operation
   */
  
  private static void addOperation (Operation op)
  {
    operations.put (op.getName (), op);
  }
  
  
  
  /**
   *  return whether the given domain name only contains ASCII characters
   * 
   *  @param name           the name
   *  @return               {@code true} if so
   */
  
  private static boolean isAscii (String name)
  {
    int length = name.length ();
    
    for (int i = 0; i < length; i++)
      if (name.charAt (i) > 127)
        return false;
    
    return true;
  }
  

  
  /**
   *  execute a command
   *
   *  @param args              the arguments
   *  @param offset            the index to the first argument to use
   *  @return                  {@code null} if command could not be parsed
   */
  
  private static Command parseCommand (String[] args, int offset)
  {
    String command = args[offset];
    String[] commandArgs = new String[args.length - offset - 1];
    
    System.arraycopy (args, offset + 1, commandArgs, 0, 
      args.length - offset - 1);

    PrintWriter pw = new PrintWriter (System.out);
    
    try
    {
      if (command.equals ("help"))
      {
        if (commandArgs.length > 0)
        {
          String commandName = commandArgs[0];
          Operation op = operations.get (commandName);

          if (op != null)
          {
            op.printHelp (pw);
            return null;
          }
        }

        pw.println ("commands:");

        for (Operation op: operations.values ())
        {
          pw.println ();
          op.printHelp (pw);
        }

        return null;

      } else
      {
        Operation op = operations.get (command);

        if (op == null)
        {
          pw.println ("unknown command \"" + command + "\". available commands:");

          for (String name: operations.keySet ())
            pw.println ("  " + name);

          return null;
        }

        Arguments a = new Arguments (commandArgs);
        Command cmd = op.parse (a, pw);

        if (cmd == null)
          op.printHelp (pw);

        return cmd;      
      }
    } finally 
    {
      pw.flush ();
    }
  }
  
  
  
  /**
   *  split a line into arguments. Quotes are observed
   *
   *  @param line             the line
   *  @return                 the parsed arguments
   */
  
  private String[] splitArgs (String line)
  {
    Matcher m = argPattern.matcher (line);
    List<String> args = new ArrayList<> ();
    int pos = 0;
    
    while (m.find (pos))
    {
      String temp = m.group (1);
      if (temp == null) temp = m.group (2);
      args.add (temp);
      
      pos = m.end ();
    }
    
    String[] argArray = new String[args.size ()];
    args.toArray (argArray);
    return argArray;
  }
  
  
  
  
  /**
   *  parse a section of the batch file
   *
   *  @param stopWords                the words to stop parsing
   *  @return                         the parsed command or {@code null}
   *                                  if end of input has been encountered
   *                                  without reading a single command
   *  @throws IOException             on exceptions while reading
   */
  
  private Command parseBatchSection (String... stopWords)
    throws IOException
  {
    List<Command> commands = new ArrayList<> ();
    
    while (true)
    {
      while (parsedArgs == null)
      {
        String line = batchReader.readLine ();
        
        if (line == null)
        {
          if (commands.isEmpty ()) 
            return null;
          
          break;
        }
        
        parsedArgs = splitArgs (line);
      }
      
      if (parsedArgs == null)
        break;
      
      if (parsedArgs.length == 0)
      {
        parsedArgs = null;
        continue;
      }
      
      String token = parsedArgs[0];
      boolean found = false;
      
      for (String temp: stopWords)
      {
        if (temp.equals (token))
        {
          found = true;
          break;
        }
      }
      
      if (found) break;
      
      if (token.equals ("[#"))
      { 
        int repeatCount = 1;
        
        if (parsedArgs.length > 1)
        {
          try
          {
            repeatCount = Integer.parseInt (parsedArgs[1]);
            
          } catch (NumberFormatException e)
          {
            // ignored
          }
        }
        
        parsedArgs = null;
        Command tempCommand = parseBatchSection ("#]");
        
        if (tempCommand != null) 
          commands.add (new RepeatCommand (tempCommand, repeatCount));
        
        parsedArgs = null;
        
      } else if (token.equals ("[|"))
      {
        int threadCount = 1;

        if (parsedArgs.length > 1)
        {
          try
          {
            threadCount = Integer.parseInt (parsedArgs[1]);
            
          } catch (NumberFormatException e)
          {
            // ignored
          }
        }
        
        List<Command> list = new ArrayList<> ();
        
        while (true)
        {
          parsedArgs = null;
          Command tempCommand = parseBatchSection ("|", "|]");
          
          if (tempCommand == null)
            break;
          
          list.add (tempCommand);
          
          if (parsedArgs == null || parsedArgs.length == 0 ||
            !parsedArgs[0].equals ("|")) 
            break;
        }
        
        Command[] c = new Command[list.size ()];
        list.toArray (c);
        commands.add (new ParallelCommand (c, threadCount));
        
      } else
      {
        // normal command
        Command cmd = parseCommand (parsedArgs, 0);
        
        if (cmd != null)
          commands.add (cmd);
      }
      
      parsedArgs = null;
    }
    
    if (commands.size () == 1) 
      return commands.get (0);
    
    Command[] c = new Command[commands.size ()];
    commands.toArray (c);
    return new SequentialCommand (c);
  }
  
  
  
  
  /**
   *  parse the given batch file
   *
   *  @param reader            the reader
   *  @return                  the command
   *  @throws IOException      on i/o exceptions
   */
  
  private Command parseBatchFile (BufferedReader reader)
    throws IOException
  {
    batchReader = reader;
    parsedArgs = null;
    
    try
    {
      return parseBatchSection ();
      
    } finally
    {
      batchReader = null;
      parsedArgs = null;
    }
  }
  
  
  /**
   *  the main program
   *
   *  @param args              the command line arguments
   */
  
  private void doMain (String[] args)
  {
    if (args.length < 2)
    {
      System.err.println ("arguments: <properties> <command> ...");
      System.err.println ();
      System.err.println ("the following commands are available:");
      
      for (String name: operations.keySet ())
        System.err.println ("  " + name);
      
      System.err.println ();
      System.err.println ("to see a detailed list of all commands, " +
        "use \"help\". For a single command");
      System.err.println ("use \"help <command>\".");
      
      return;
    }
    
    try
    {
      Properties props = new Properties ();
      props.load (new FileInputStream (args[0]));
      
      TcpParams params = new TcpParams ();
      params.setupFromProperties (props,  "tcp.");
      ChannelFactory factory = new TcpChannelFactory (params);
      
      factory = new DebugChannelFactory (factory);
      
      String temp = props.getProperty ("xml");
      
      boolean useXml = "yes".equalsIgnoreCase (temp) || 
        "true".equalsIgnoreCase (temp);
      
      String userName = props.getProperty ("user");
      String password = props.getProperty ("password");

      Codec codec = new Codec (DefaultResponseFactory.getFactory (), useXml);

      String command = args[1];
      Command cmd;
      
      if (command.startsWith ("@"))
      {
        String filename = command.substring (1);
        BufferedReader reader;
        
        if (filename.equals ("-"))
          reader = new BufferedReader (new InputStreamReader (System.in));
         else
          reader = new BufferedReader (new FileReader (filename));
        
        cmd = parseBatchFile (reader);
        
      } else
      {
        cmd = parseCommand (args, 1);
      }
      
      if (cmd != null)
      {
        PooledPort pport = new PooledPort (factory, codec, userName, password, 
          1, 2, 100);

        Port port = new DebugPort (pport);

        long start = System.currentTimeMillis ();
        cmd.execute (port);
        long end = System.currentTimeMillis ();

        System.err.println ("execution time: " + (end - start) + "ms");
  
        pport.shutdown ();
      }
    } catch (Exception e)
    {
      e.printStackTrace (System.err);
    }
  }
  
  
  
  /**
   *  main program
   *
   *  @param args              the command line arguments
   */
  
  public static void main (String[] args)
  {
    Test t = new Test ();
    t.doMain (args);
  }
  
  

  /**
   *  small interface to a class that executes commands on a given port
   *
   *  @version  $Revision: 1000 $
   *  @author   Klaus Malorny
   */
  
  private interface Command
  {
    /**
     *  executes the command
     *
     *  @param port                   the port
     *  @throws Exception             if the execution failed for some reason
     */
    
    public void execute (Port port)
      throws Exception;
  }
  
  
  
  /**
   *  class that repeats a command a given number of times
   *
   *  @version  $Revision: 1000 $
   *  @author   Klaus Malorny
   */
  
  private static class RepeatCommand implements Command
  {
    /** the command to repeat */
    
    private Command command;
    
    
    /** the number of repetitions */
    
    private int repeatCount;
    
    
    /**
     *  constructor
     *
     *  @param theCommand                  the command
     *  @param theRepeatCount              the number of repetitions
     */
    
    public RepeatCommand (Command theCommand, int theRepeatCount)
    {
      command = theCommand;
      repeatCount = theRepeatCount;
    }

    /**
     *  execute the command
     *
     *  @param port                   the port
     *  @throws Exception             if the execution failed for some reason
     */

    @Override
    public void execute (Port port)
      throws Exception
    {
      for (int i = 0; i < repeatCount; i++)
        command.execute (port);
    }
  }
  
  
  /**
   *  class that executes the given commands sequentially
   *
   *  @version  $Revision: 1000 $
   *  @author   Klaus Malorny
   */
  
  private static class SequentialCommand implements Command
  {
    /** the commands to execute */
    
    private Command[] commands;
    
    
    /**
     *  constructor
     *
     *  @param theCommands             the commands to execute
     */
    
    public SequentialCommand (Command[] theCommands)
    {
      commands = theCommands;
    }

    
    
    /**
     *  execute the command
     *
     *  @param port                   the port
     *  @throws Exception             if the execution failed for some reason
     */

    @Override
    public void execute (Port port)
      throws Exception
    {
      for (Command command: commands)
        command.execute (port);
    }
  }
  
  
  
  /**
   *  class that executes the given commands in parallel
   *
   *  @version   $Revision: 1000 $
   *  @author    Klaus Malorny
   */
  
  private static class ParallelCommand implements Command
  {
    /** the commands to execute */
    
    private Command[] commands;
    
    
    /** the number of threads to use */
    
    private int threadCount;
    
    
    /**
     *  constructor
     *
     *  @param theCommands             the commands to execute
     *  @param theThreadCount          the maximum number of threads to use
     */
    
    public ParallelCommand (Command[] theCommands, int theThreadCount)
    {
      commands = theCommands;
      threadCount = theThreadCount;
    }
    
    
    
    /**
     *  execute the command
     *
     *  @param port                   the port
     *  @throws Exception             if the execution failed for some reason
     */

    @Override
    public void execute (final Port port)
      throws Exception
    {
      if (commands.length == 0)
        return;
    
      final Queue<Command> queue = new ConcurrentLinkedQueue<> ();
      Collections.addAll (queue, commands);
      
      int count = Math.min (threadCount, commands.length);
      
      Runnable r = () ->
        {
          Command command;

          while ((command = queue.poll ()) != null)
          {
            try
            {
              command.execute (port);
            } catch (Exception e)
            {
              e.printStackTrace (System.err);
            }
          }
        };
    
      Thread[] threads = new Thread[count - 1];
      
      for (int i = 0; i < count - 1; i++)
       threads[i] = new Thread (r);
       
      for (Thread t: threads)
        t.start ();
      
      r.run ();
      
      for (Thread t: threads)
        t.join ();
    }
  }  
    
  

  /**
   *  small class to wrap the arguments of a command
   * 
   *  @version  $Revision: 1000 $
   *  @author   Klaus Malorny
   */
  
  private static class Arguments
  {
    /** the array containing the arguments */
    
    private String[] args;
    
    
    /** the reading position */
    
    private int pos;
    
    
    
    /**
     *  constructor
     * 
     *  @param theArgs        the arguments
     */
    
    public Arguments (String[] theArgs)
    {
      args = theArgs;
      pos = 0;
    }
    
    
    
    /**
     *  return whether more arguments are available
     * 
     *  @return               {@code true} if so
     */
    
    public boolean hasMore ()
    {
      return pos < args.length;
    }
    
    
    
    /**
     *  return whether an argument exists and this is an option (i.e.
     *  starting with a hyphen)
     * 
     *  @return               {@code true} if so
     */
    
    public boolean isOption ()
    {
      if (pos >= args.length)
        return false;
      
      String arg = args[pos];
      return arg.length () > 0 && arg.charAt (0) == '-';
    }
    
    
    /**
     *  return whether an argument exists and is not a value (i.e. not
     *  starting with a hyphen)
     * 
     *  @return               {@code true} if so
     */
    
    public boolean isValue ()
    {
      if (pos >= args.length)
        return false;
      
      String arg = args[pos];
      return arg.length () == 0 || arg.charAt (0) != '-';
    }
    
    
    
    /**
     *  return the current argument
     * 
     *  @return               the current argument or {@code null} if there
     *                        are no more arguments
     */
    
    public String getArg ()
    {
      return pos < args.length ? args[pos] : null;
    }
    
    
    
    /**
     *  return the current argument, moving the internal position to the next
     *  argument.
     * 
     *  @return               the next argument or {@code null} if the
     *                        end of the arguments have been reached
     */
    
    public String nextArg ()
    {
      return pos < args.length ? args[pos++] : null;
    }
    
    
    
    /**
     *  return the next value and moving the internal position to the next
     *  argument. If the current argument is not a value, {@code null} is
     *  returned and the position is left unchanged
     * 
     *  @return               the next option or {@code null} if no more
     *                        values are available or if the current argument
     *                        is not a value
     */
    
    public String nextValue ()
    {
      return isValue () ? args[pos++] : null;
    }
    
    
    /**
     *  advance the position to the next argument, unless there are no more
     */
    
    public void advance ()
    {
      if (pos < args.length)
        pos++;
    }
    
  }
  
  
  
  /**
   *  interface to an operation
   * 
   *  @version  $Revision: 1000 $
   *  @author   Klaus Malorny
   */
  
  private interface Operation
  {
    /**
     *  return the name of the operation
     * 
     *  @return               the name
     */
    
    public String getName ();
    
    
    
    /**
     *  print the help for the operation; the output shall not exceed
     *  80 characters per line
     * 
     *  @param pw             the print writer to use
     */
    
    public void printHelp (PrintWriter pw);
    
    
    
    /**
     *  create a command from the arguments; note that in case of errors,
     *  no help shall be printed (i.e. call to {@link Operation#printHelp}
     * 
     *  @param args           the arguments
     *  @param pw             the print writer to use for error output
     *  @return               the command or {@code null} if the arguments
     *                        could not be parsed
     */
    
    public Command parse (Arguments args, PrintWriter pw);
  }
  
  
  
  
  /**
   *  common base class for operations with some helpers
   * 
   *  @version  $Revision: 1000 $
   *  @author   Klaus Malorny
   */
  
  private abstract static class BaseOperation implements Operation
  {
    /**
     *  print an error that the option may not be repeated
     * 
     *  @param pw             the print writer to use for error output
     *  @param opt            the option
     */
    
    protected void printRepeatedOptionError (PrintWriter pw, String opt)
    {
      pw.println (opt + ": already specified");
    }
    
    
    
    /**
     *  try to parse a single value option
     * 
     *  @param args           the arguments
     *  @param pw             the print writer to use for error output
     *  @param opt            the option for errors
     *  @param descr          the description of the argument, for errors
     *  @return               the value or {@code null}
     */
    
    protected String parseSingleValueOption (Arguments args, PrintWriter pw,
      String opt, String descr)
    {
      String value = args.nextValue ();

      if (value == null)
      {
        pw.println (opt + ": " + descr + " expected");
        return null;
      }

      return value;
    }
    


    
    
    /**
     *  try to parse a single value option, which must be an integer value
     * 
     *  @param args           the arguments
     *  @param pw             the print writer to use for error output
     *  @param opt            the option for errors
     *  @param descr          the description of the argument, for errors
     *  @return               the value or {@code null}
     */
    
    protected Integer parseSingleIntValueOption (Arguments args, PrintWriter pw,
      String opt, String descr)
    {
      String temp = parseSingleValueOption (args, pw, opt, descr);
      
      if (temp == null)
        return null;
      
      try
      {
        return Integer.valueOf (temp);
        
      } catch (NumberFormatException e)
      {
        pw.println (opt + ": \"" + temp + "\" is not a valid number for the " +
          descr);
        
        return null;
      }
    }
    

    
    /**
     *  try to parse an option with one or more values
     * 
     *  @param args           the arguments
     *  @param pw             the print writer to use for error output
     *  @param opt            the option for errors
     *  @param descr          the description of the argument, for errors
     *  @param consumer       the consumer receiving the values
     *  @return               {@code true} if successful
     */
    
    protected boolean parseMultipleValueOption (Arguments args, PrintWriter pw,
      String opt, String descr, Consumer<String> consumer)
    {
      String value = args.nextValue ();

      if (value == null)
      {
        pw.println (opt + ": " + descr + " expected");
        return false;
      }
      
      do
      {
        consumer.accept (value);
        
      } while ((value = args.nextValue ()) != null);
      
      return true;
    }
  }
    
  
  
  
  
  /**
   *  common class for contact create and update
   * 
   *  @version  $Revision: 1000 $
   *  @author   Klaus Malorny
   */
  
  private static class ContactCU extends BaseOperation
  {
    /** the operation name */
    
    private String opName;
    
    
    /** the request factory */
    
    private Function<ContactData, Request> requestFactory;
    
    
    /**
     *  constructor
     * 
     *  @param name           the operation name
     *  @param factory        the factory
     */
    
    public ContactCU (String name, Function<ContactData, Request> factory)
    {
      opName = name;
      requestFactory = factory;
    }
    
    
    
    /**
     *  parse the arguments
     * 
     *  @param args           the arguments
     *  @param pw             the print writer to use for error output
     *  @return               the request
     */
    
    protected Request parseData (Arguments args, PrintWriter pw)
    {
      boolean success = true;
      String handle = null;
      String name = null;
      List<String> org = new ArrayList<> ();
      List<String> addr = new ArrayList<> ();
      String pc = null;
      String city = null;
      String cc = null;
      List<String> email = new ArrayList<> ();
      String uri = null;
      ContactData.Type type = null;
      
      while (args.hasMore ())
      {
        if (!args.isOption ())
        {
          pw.println ("option expected");
          return null;
        }
        
        String option = args.nextArg ();
        
        switch (option)
        {
          case "-h":
            if (handle != null)
            {
              printRepeatedOptionError (pw, option);
              success = false;
            
            } else
            {
              handle = parseSingleValueOption (args, pw, option, "handle");
              
              if (handle == null)
                success = false;
            }

            break;
            
          case "-t":
            if (type != null)
            {
              printRepeatedOptionError (pw, option);
              success = false;
            
            } else
            {
              String temp = parseSingleValueOption (args, pw, option, "type");
              
              if (temp == null)
              {
                success = false;
                
              } else
              {
                type = ContactData.Type.fromToken (temp);

                if (type == null)
                {
                  pw.println (option + ": unknown type \"" + temp + "\"");
                  success = false;
                  break;
                }
              }
            }

            break;
            
          case "-n":
            if (name != null)
            {
              printRepeatedOptionError (pw, option);
              success = false;
            
            } else
            {
              name = parseSingleValueOption (args, pw, option, "name");
              
              if (name == null)
                success = false;
            }

            break;
            
          case "-o":
            success &= parseMultipleValueOption (args, pw, option, 
              "organization", org::add);
            
            break;
            
          case "-a":
            success &= parseMultipleValueOption (args, pw, option, "address",
              addr::add);
            
            break;
            
          case "-pc":
            if (pc != null)
            {
              printRepeatedOptionError (pw, option);
              success = false;
            
            } else
            {
              pc = parseSingleValueOption (args, pw, option, "postal code");
              
              if (pc == null)
                success = false;
            }

            break;
            
          case "-c":
            if (city != null)
            {
              printRepeatedOptionError (pw, option);
              success = false;
            
            } else
            {
              city = parseSingleValueOption (args, pw, option, "city");
              
              if (city == null)
                success = false;
            }

            break;
            
          case "-cc":
            if (cc != null)
            {
              printRepeatedOptionError (pw, option);
              success = false;
            
            } else
            {
              cc = parseSingleValueOption (args, pw, option, "country code");
              
              if (cc == null)
                success = false;
            }

            break;
            
          case "-e":
            success &= parseMultipleValueOption (args, pw, option, 
              "e-mail address", email::add);
            
            break;
            
          case "-u":
            if (uri != null)
            {
              printRepeatedOptionError (pw, option);
              success = false;
            
            } else
            {
              uri = parseSingleValueOption (args, pw, option, "URI template");
              
              if (uri == null)
                success = false;
            }

            break;
            
          default:
            pw.println (option + ": unknown option");
            success = false;
            break;
        }
      }
      
      if (handle == null)
      {
        pw.println ("handle must be specified");
        success = false;
      }
      
      if (type == null)
      {
        pw.println ("type must be specified");
        success = false;
      }
      
      if (!success)
        return null;
      
      ContactData cd = new ContactData ();
      
      cd.setHandle (handle);
      cd.setType (type);
      cd.setName (name);
      cd.setOrganisation (org);
      cd.setAddress (addr);
      cd.setPostalCode (pc);
      cd.setCity (city);
      cd.setCountryCode (cc);
      cd.setEMailAddresses (email);
      cd.setUriTemplate (uri);
        
      return requestFactory.apply (cd);
    }
    
    
    
    /**
     *  return the name of the operation
     * 
     *  @return               the name
     */
    
    @Override
    public String getName ()
    {
      return opName;
    }
    
    
    
    /**
     *  print the help for the operation; the output shall not exceed
     *  80 characters per line
     * 
     *  @param pw             the print writer to use
     */
    
    @Override
    public void printHelp (PrintWriter pw)
    {
      pw.println (opName + " -h <handle> -t <type> [ -n <name> ] " +
        "[ -o <org> ... ]\n  [ -a <addr> ... ] [ -pc <postal code> ] " +
        "[ -c <city> ]\n  [ -cc <country code> ] [ -e <email> ... ] " +
        "[ -u <uri template> ]");
      
      pw.println ();
      pw.println ("  -h <handle>             the handle");
      pw.println ("  -t <type>               the contact type, may be one of:");
      
      for (ContactData.Type type: ContactData.Type.values ())
        pw.println ("                            " + type.getDenicToken ());

      pw.println ("  -n <name>               name");
      pw.println ("  -o <org> ...            organization lines");
      pw.println ("  -a <addr> ...           address lines");
      pw.println ("  -pc <postal code>       postal code");
      pw.println ("  -c <city>               city");
      
      pw.println ("  -cc <country code>      country code (ISO 3166 " +
        "two letter)");
      
      pw.println ("  -e <email>              email addresses");
      pw.println ("  -u <uri template>       URI template");
    }
    
    
    
    /**
     *  create a command from the arguments; note that in case of errors,
     *  no help shall be printed (i.e. call to {@link Operation#printHelp}
     * 
     *  @param args           the arguments
     *  @param pw             the print writer to use for error output
     *  @return               the command or {@code null} if the arguments
     *                        could not be parsed
     */
    
    @Override
    public Command parse (Arguments args, PrintWriter pw)
    {
      Request req = parseData (args, pw);
      
      if (req == null)
        return null;
      
      return req::execute;
    }
  }
  
  
  
  /**
   *  contact check/info operation
   * 
   *  @version  $Revision: 1000 $
   *  @author   Klaus Malorny
   */
  
  private static class ContactCI extends BaseOperation
  {
    /** the operation name */
    
    private String opName;
    
    
    /** the request factory */
    
    private Function<String, Request> requestFactory;
    
    
    /**
     *  constructor
     * 
     *  @param name           the operation name
     *  @param factory        the factory
     */
    
    public ContactCI (String name, Function<String, Request> factory)
    {
      opName = name;
      requestFactory = factory;
    }
    
    /**
     *  return the name of the operation
     * 
     *  @return               the name
     */
    
    @Override
    public String getName ()
    {
      return opName;
    }
    
    
    
    /**
     *  print the help for the operation; the output shall not exceed
     *  80 characters per line
     * 
     *  @param pw             the print writer to use
     */
    
    @Override
    public void printHelp (PrintWriter pw)
    {
      pw.println (opName + " -h <handle>");
      pw.println ();
      pw.println ("  -h <handle>             the handle");
    }
    
    
    
    /**
     *  create a command from the arguments; note that in case of errors,
     *  no help shall be printed (i.e. call to {@link Operation#printHelp}
     * 
     *  @param args           the arguments
     *  @param pw             the print writer to use for error output
     *  @return               the command or {@code null} if the arguments
     *                        could not be parsed
     */
    
    @Override
    public Command parse (Arguments args, PrintWriter pw)
    {
      String handle = null;
      boolean success = true;
    
      while (args.hasMore ())
      {
        if (!args.isOption ())
        {
          pw.println ("option expected");
          return null;
        }
        
        String option = args.nextArg ();
        
        switch (option)
        {
          case "-h":
            if (handle != null)
            {
              printRepeatedOptionError (pw, option);
              success = false;
            
            } else
            {
              handle = parseSingleValueOption (args, pw, option, "handle");
              
              if (handle == null)
                success = false;
            }

            break;
            
          default:
            pw.println (option + ": unknown option");
            success = false;
            break;
        }
      }
      
      if (handle == null)
      {
        pw.println ("missing handle");
        success = false;
      }
      
      
      if (!success)
        return null;
      
      String finalHandle = handle;
      
      return port ->
        {
          Request req = requestFactory.apply (finalHandle);
          Response resp = req.execute (port);
          System.out.println (resp);
        };
    }
  }
  
  
  /**
   *  common class for domain operations requiring the full data
   * 
   *  @version  $Revision: 1000 $
   *  @author   Klaus Malorny
   */
  
  private static class DomainCU extends BaseOperation
  {
    /** the operation name */
    
    private String opName;
    
    
    /** flag whether authinfo is supported/required */
    
    private boolean withAuthInfo;
    
    
    /** the request factory */
    
    private Function<DomainData, Request> requestFactory;
    
    
    /**
     *  constructor
     * 
     *  @param name           the operation name
     *  @param factory        the factory
     *  @param authInfo       {@code true} if authinfo is supported/required
     */
    
    public DomainCU (String name, Function<DomainData, Request> factory,
      boolean authInfo)
    {
      opName = name;
      requestFactory = factory;
      withAuthInfo = authInfo;
    }
    
    
    
    /**
     *  parse the arguments
     * 
     *  @param args           the arguments
     *  @param pw             the print writer to use for error output
     *  @return               the request or {@code null} in case of errors
     */
    
    protected Request parseData (Arguments args, PrintWriter pw)
    {
      boolean success = true;
      String uName = null;
      String aName = null;
      List<String> holders = new ArrayList<> ();
      String generalRequest = null;
      String abuseContact = null;
      List<DnsEntry> dns = new ArrayList<> ();
      String authInfo = null;
      
      while (args.hasMore ())
      {
        if (!args.isOption ())
        {
          pw.println ("option expected");
          return null;
        }
        
        String option = args.nextArg ();
        
        switch (option)
        {
          case "-dn":
            if (uName != null)
            {
              pw.println (option + ": domain name already specified");
              success = false;
            
            } else
            {
              uName = args.nextValue ();
              
              if (uName == null)
              {
                pw.println (option + ": domain name expected");
                success = false;
              }
              
              aName = args.nextValue ();
              
              if (aName == null && !isAscii (uName))
                aName = IDN.toASCII (uName);
                  // note: this is IDNA2003, while DENIC uses IDNA2008
            }

            break;
            
          case "-h":
            success &= parseMultipleValueOption (args, pw, option, "holder",
              holders::add);
            
            break;
            
          case "-gr":
            if (generalRequest != null)
            {
              printRepeatedOptionError (pw, option);
              success = false;
              
            } else
            {
              generalRequest = parseSingleValueOption (args, pw, option, 
                "general request contact");
              
              if (generalRequest == null)
                success = false;
            }
            
            break;
            
          case "-ac":
            if (abuseContact != null)
            {
              printRepeatedOptionError (pw, option);
              success = false;
              
            } else
            {
              abuseContact = parseSingleValueOption (args, pw, option, 
                "abuse contact");
              
              if (abuseContact == null)
                success = false;
            }
            
            break;
    
          case "-ns":
            String hostName = args.nextValue ();
            
            if (hostName == null)
            {
              pw.println (option + ": name server name expected");
              success = false;
              break;
            }
            
            List<String> ips = new ArrayList<> ();
            
            while (args.isValue ())
              ips.add (args.nextArg ());
            
            dns.add (new DnsEntryNS (hostName, ips));
            break;

          case "-a":
          case "-aaaa":
            String ownerName = args.nextValue ();
            
            if (ownerName == null)
            {
              pw.println (option + ": owner name expected");
              success = false;
              break;
            }
            
            String ip = args.nextValue ();
            
            if (ip == null)
            {
              pw.println (option + ": IP address expected");
              success = false;
              break;
            }
            
            dns.add (option.equals ("-a") ? new DnsEntryA (ownerName, ip) :
              new DnsEntryAAAA (ownerName, ip));
            
            break;
            
          case "-mx":
            ownerName = args.nextValue ();
            
            if (ownerName == null)
            {
              pw.println (option + ": owner name expected");
              success = false;
              break;
            }
            
            String temp = args.nextValue ();
            
            if (temp == null)
            {
              pw.println (option + ": priority expected");
              success = false;
              break;
            }
            
            int priority;
            
            try
            {
              priority = Integer.parseInt (temp);
              
            } catch (NumberFormatException e)
            {
              pw.println (option + ": invalid priority \"" + temp + "\"");
              success = false;
              break;
            }
            
            String exchangeName = args.nextValue ();
            
            if (exchangeName == null)
            {
              pw.println (option + ": mail exchange name expected");
              success = false;
              break;
            }
            
            dns.add (new DnsEntryMX (ownerName, priority, exchangeName));
            break;
            
          case "-dnskey":
            temp = args.nextValue ();
            
            if (temp == null)
            {
              pw.println (option + ": flags expected");
              success = false;
              break;
            }
            
            int flags;
            
            try
            {
              flags = Integer.parseInt (temp);
              
            } catch (NumberFormatException e)
            {
              pw.println (option + ": invalid flags \"" + temp + "\"");
              success = false;
              break;
            }
            
            temp = args.nextValue ();
            
            if (temp == null)
            {
              pw.println (option + ": protocol expected");
              success = false;
              break;
            }
            
            int protocol;
            
            try
            {
              protocol = Integer.parseInt (temp);
              
            } catch (NumberFormatException e)
            {
              pw.println (option + ": invalid protocol \"" + temp + "\"");
              success = false;
              break;
            }
            
            temp = args.nextValue ();
            
            if (temp == null)
            {
              pw.println (option + ": algorithm expected");
              success = false;
              break;
            }
            
            int algorithm;
            
            try
            {
              algorithm = Integer.parseInt (temp);
              
            } catch (NumberFormatException e)
            {
              pw.println (option + ": invalid algorithm \"" + temp + "\"");
              success = false;
              break;
            }
            
            String key = args.nextValue ();
            
            if (key == null)
            {
              pw.println (option + ": key expected");
              success = false;
              break;
            }
            
            dns.add (new DnsEntryDNSKEY (flags, protocol, algorithm, key));
            break;
            
          case "-ai":
            if (withAuthInfo)
            {
              if (authInfo != null)
              {
                printRepeatedOptionError (pw, option);
                success = false;
                
              } else
              {
                authInfo = args.nextValue ();
                
                if (authInfo == null)
                {
                  pw.println (option + ": authinfo expected");
                  success = false;
                }
              }
            } else
            {
              pw.println (option + ": unknown option");
              success = false;
            }
            
            break;
            
          default:
            pw.println (option + ": unknown option");
            success = false;
        }
      }
      
      if (uName == null)
      {
        pw.println ("domain name must be specified");
        success = false;
      }
      
      if (withAuthInfo && authInfo == null)
      {
        pw.println ("authinfo must be specified");
        success = false;
      }
      
      if (!success)
        return null;
      
      DomainData dd = new DomainData ();
      
      if (aName != null)
        dd.setDomainName (uName, aName);
       else
        dd.setDomainName (uName);
      
      for (String holder: holders)
      {
        ContactData cd = new ContactData ();
        cd.setHandle (holder);
        dd.addContact (Role.Holder, cd);
      }
      
      if (generalRequest != null)
      {
        ContactData cd = new ContactData ();
        cd.setHandle (generalRequest);
        dd.addContact (Role.GeneralRequest, cd);
      }
      
      if (abuseContact != null)
      {
        ContactData cd = new ContactData ();
        cd.setHandle (abuseContact);
        dd.addContact (Role.AbuseContact, cd);
      }
      
      dd.addDnsEntries (dns);
      dd.setChangeProvAuthInfo (authInfo);

      return requestFactory.apply (dd);      
    }
    
    
    
    /**
     *  return the name of the operation
     * 
     *  @return               the name
     */
    
    @Override
    public String getName ()
    {
      return opName;
    }
    
    
    
    /**
     *  print the help for the operation; the output shall not exceed
     *  80 characters per line
     * 
     *  @param pw             the print writer to use
     */
    
    @Override
    public void printHelp (PrintWriter pw)
    {
      pw.println (opName + " -dn <u-name> [ <a-name> ]\n" +
        "  [ -h <handle> ... ] [ -gr <handle> ] [ -ac <handle> ]\n" +
        "  [ -ns <name> [ <ip> ... ] ] ... [ -a <owner> <ip> ] ...\n" +
        "  [ -aaaa <owner> <ip> ] ... [ -mx <owner> <priority> " +
        "<exchange> ] ...\n  [ -dnskey <flags> <protocol> " +
        "<algorithm> <key> ] ... ");
      
      if (withAuthInfo)
        pw.println ("  -ai <authinfo>");
      
      pw.println ();
      
      pw.println ("  -dn <u-name>            the domain name; if a-name is " +
        "omitted, it is");
      
      pw.println ("     [ <a-name> ]         generated (due to lack of JRE " +
        "support, IDNA2003");
      
      pw.println ("                          is used, while DENIC uses " +
        "IDNA2008)");
      pw.println ("  -h <holder> ...         one or more holder contact " +
        "handles");
      
      pw.println ("  -gr <handle>            general request contact handle");
      pw.println ("  -ac <handle>            abuse contact handle");

      pw.println ("  -ns <name> [ <ip> ... ] name server name with optional " +
        "IP addresses for ");

      pw.println ("                          in-zone hosts, may be repeated");
      pw.println ("  -a <owner> <ip>         A record, may be repeated");
      pw.println ("  -aaaa <owner> <ip>      AAAA record, may be repeated");
      pw.println ("  -mx <owner> <priority>  MX record, may be repeated");
      pw.println ("     <exchange>");
      
      pw.println ("  -dnskey <flags>         DNSKEY record; key must be " +
        "specified as Base64");
      
      pw.println ("    <protocol> <algorithm>");
      pw.println ("    <key>");

      if (withAuthInfo)
        pw.println ("  -ai <authinfo>          the authinfo");
    }
    
    
    
    /**
     *  create a command from the arguments; note that in case of errors,
     *  no help shall be printed (i.e. call to {@link Operation#printHelp}
     * 
     *  @param args           the arguments
     *  @param pw             the print writer to use for error output
     *  @return               the command or {@code null} if the arguments
     *                        could not be parsed
     */
    
    @Override
    public Command parse (Arguments args, PrintWriter pw)
    {
      Request req = parseData (args, pw);
      
      if (req == null)
        return null;
      
      return req::execute;
    }
  }

  
  
  /**
   *  common class for domain operations requiring only the domain name
   * 
   *  @param <T>           the request type
   * 
   *  @version  $Revision: 1000 $
   *  @author   Klaus Malorny
   */
  
  private static class DomainSimple<T extends Request> extends BaseOperation
  {
    /** the operation name */
    
    private String opName;
    
    
    /** the request factory */
    
    private BiFunction<String, String, T> requestFactory;
    
    
    /**
     *  constructor
     * 
     *  @param name           the operation name
     *  @param factory        the factory
     */
    
    public DomainSimple (String name, 
      BiFunction<String, String, T> factory)
    {
      opName = name;
      requestFactory = factory;
    }
    
    
    
    /**
     *  parse the arguments
     * 
     *  @param args           the arguments
     *  @param pw             the print writer to use for error output
     *  @return               the request or {@code null} if the arguments
     *                        could not be parsed
     */
    
    protected T parseData (Arguments args, PrintWriter pw)
    {
      boolean success = true;
      String uName = null;
      String aName = null;
      
      while (args.hasMore ())
      {
        if (!args.isOption ())
        {
          pw.println ("option expected");
          return null;
        }
        
        String option = args.nextArg ();
        
        switch (option)
        {
          case "-dn":
            if (uName != null)
            {
              pw.println (option + ": domain name already specified");
              success = false;
            
            } else
            {
              uName = args.nextValue ();
              
              if (uName == null)
              {
                pw.println (option + ": domain name expected");
                success = false;
              }
              
              aName = args.nextValue ();
              
              if (aName == null && !isAscii (uName))
                aName = IDN.toASCII (uName);
                  // note: this is IDNA2003, while DENIC uses IDNA2008
            }

            break;
            
          default:
            pw.println (option + ": unknown option");
            success = false;
        }
      }
      
      if (uName == null)
      {
        pw.println ("domain name must be specified");
        success = false;
      }
      
      if (!success)
        return null;
      
      return requestFactory.apply (uName, aName);
    }
    
    
    
    /**
     *  return the name of the operation
     * 
     *  @return               the name
     */
    
    @Override
    public String getName ()
    {
      return opName;
    }
    
    
    
    /**
     *  print the help for the operation; the output shall not exceed
     *  80 characters per line
     * 
     *  @param pw             the print writer to use
     */
    
    @Override
    public void printHelp (PrintWriter pw)
    {
      pw.println (opName + " -dn <u-name> [ <a-name> ]");
      
      pw.println ();
      
      pw.println ("  -dn <u-name>            the domain name; if a-name is " +
        "omitted, it is");
      
      pw.println ("     [ <a-name> ]         generated (due to lack of JRE " +
        "support, IDNA2003");
      
      pw.println ("                          is used, while DENIC uses " +
        "IDNA2008)");
    }
    
    
    
    /**
     *  create a command from the arguments; note that in case of errors,
     *  no help shall be printed (i.e. call to {@link Operation#printHelp}
     * 
     *  @param args           the arguments
     *  @param pw             the print writer to use for error output
     *  @return               the command or {@code null} if the arguments
     *                        could not be parsed
     */
    
    @Override
    public Command parse (Arguments args, PrintWriter pw)
    {
      Request req = parseData (args, pw);
      
      if (req == null)
        return null;
      
      return req::execute;
    }
  }
    
  
  
  
  
  /**
   *  operation to delete a domain
   * 
   *  @version  $Revision: 1000 $
   *  @author   Klaus Malorny
   */
  
  private static class DomainDelete extends DomainSimple<DomainInfoRequest>
  {
    /**
     *  constructor
     */
    
    public DomainDelete ()
    {
      super ("domain-delete", DomainInfoRequest::new);
    }
    
    
    
    
    /**
     *  create a command from the arguments; note that in case of errors,
     *  no help shall be printed (i.e. call to {@link Operation#printHelp}
     * 
     *  @param args           the arguments
     *  @param pw             the print writer to use for error output
     *  @return               the command or {@code null} if the arguments
     *                        could not be parsed
     */
    
    @Override
    public Command parse (Arguments args, PrintWriter pw)
    {
      DomainInfoRequest ireq = parseData (args, pw);
      
      if (ireq == null)
        return null;
      
      return port ->
        {
          DomainInfoResponse iresp = ireq.execute (port);
          System.out.println (iresp);
    
          if (!iresp.isSuccess ())
          {
            System.out.println ("could not retrieve domain information");
            return;
          }

          DomainData infoData = iresp.getDomain ();
          DomainData deleteData = new DomainData ();
          
          deleteData.setDomainName (infoData.getDomainName (), 
            infoData.getEncodedDomainName ());
          
          deleteData.addContacts (Role.Holder, 
            infoData.getContacts (Role.Holder));
          
          DomainDeleteRequest dreq = new DomainDeleteRequest (deleteData);
          Response dresp = dreq.execute (port);
          System.out.println (dresp);
        };
        
    }
  }
    


  
  /**
   *  operation to retrieve information about a domain
   * 
   *  @version  $Revision: 1000 $
   *  @author   Klaus Malorny
   */
  
  private static class DomainInfo extends BaseOperation
  {
    /**
     *  parse the arguments
     * 
     *  @param args           the arguments
     *  @param pw             the print writer to use for error output
     *  @return               the request or {@code null} if the arguments
     *                        could not be parsed
     */
    
    protected Request parseData (Arguments args, PrintWriter pw)
    {
      boolean success = true;
      String uName = null;
      String aName = null;
      String authInfo = null;
      boolean recursion = false;
      
      while (args.hasMore ())
      {
        if (!args.isOption ())
        {
          pw.println ("option expected");
          return null;
        }
        
        String option = args.nextArg ();
        
        switch (option)
        {
          case "-dn":
            if (uName != null)
            {
              pw.println (option + ": domain name already specified");
              success = false;
            
            } else
            {
              uName = args.nextValue ();
              
              if (uName == null)
              {
                pw.println (option + ": domain name expected");
                success = false;
              }
              
              aName = args.nextValue ();
              
              if (aName == null && !isAscii (uName))
                aName = IDN.toASCII (uName);
                  // note: this is IDNA2003, while DENIC uses IDNA2008
            }

            break;
            
          case "-ai":
            if (authInfo != null)
            {
              printRepeatedOptionError (pw, option);
              success = false;

            } else
            {
              authInfo = args.nextValue ();

              if (authInfo == null)
              {
                pw.println (option + ": authinfo expected");
                success = false;
              }
            }

            break;
            
          case "-r":
            if (recursion)
              printRepeatedOptionError (pw, option);
             else
              recursion = true;
            
            break;
            
          default:
            pw.println (option + ": unknown option");
            success = false;
        }
      }
      
      if (uName == null)
      {
        pw.println ("domain name must be specified");
        success = false;
      }
      
      if (!success)
        return null;
      
      return new DomainInfoRequest (uName, aName, recursion, authInfo);
    }
    
    
    
    /**
     *  return the name of the operation
     * 
     *  @return               the name
     */
    
    @Override
    public String getName ()
    {
      return "domain-info";
    }
    
    
    
    /**
     *  print the help for the operation; the output shall not exceed
     *  80 characters per line
     * 
     *  @param pw             the print writer to use
     */
    
    @Override
    public void printHelp (PrintWriter pw)
    {
      pw.println (getName () + " -dn <u-name> [ <a-name> ] [ -ai <authinfo> ]" +
        " [ -r ]");
      
      pw.println ();
      
      pw.println ("  -dn <u-name>            the domain name; if a-name is " +
        "omitted, it is");
      
      pw.println ("     [ <a-name> ]         generated (due to lack of JRE " +
        "support, IDNA2003");
      
      pw.println ("                          is used, while DENIC uses " +
        "IDNA2008)");
      
      pw.println ("  -ai <authinfo>          use the given authinfo to query " +
        "domains from");
      
      pw.println ("                          other regaccs");
      pw.println ("  -r                      use recursion");
    }
    
    
    
    /**
     *  create a command from the arguments; note that in case of errors,
     *  no help shall be printed (i.e. call to {@link Operation#printHelp}
     * 
     *  @param args           the arguments
     *  @param pw             the print writer to use for error output
     *  @return               the command or {@code null} if the arguments
     *                        could not be parsed
     */
    
    @Override
    public Command parse (Arguments args, PrintWriter pw)
    {
      Request req = parseData (args, pw);
      
      if (req == null)
        return null;
      
      return req::execute;
    }
  }
    
  


  
  /**
   *  operation to set the authinfo 1
   * 
   *  @version  $Revision: 1000 $
   *  @author   Klaus Malorny
   */
  
  private static class AuthInfo1Create extends BaseOperation
  {
    
    
    
    /**
     *  parse the arguments
     * 
     *  @param args           the arguments
     *  @param pw             the print writer to use for error output
     *  @return               the request or {@code null} if the arguments
     *                        could not be parsed
     */
    
    protected Request parseData (Arguments args, PrintWriter pw)
    {
      boolean success = true;
      String uName = null;
      String aName = null;
      Instant date = null;
      Duration duration = null;
      String authInfo = null;
      
      while (args.hasMore ())
      {
        if (!args.isOption ())
        {
          pw.println ("option expected");
          return null;
        }
        
        String option = args.nextArg ();
        
        switch (option)
        {
          case "-dn":
            if (uName != null)
            {
              pw.println (option + ": domain name already specified");
              success = false;
            
            } else
            {
              uName = args.nextValue ();
              
              if (uName == null)
              {
                pw.println (option + ": domain name expected");
                success = false;
              }
              
              aName = args.nextValue ();
              
              if (aName == null && !isAscii (uName))
                aName = IDN.toASCII (uName);
                  // note: this is IDNA2003, while DENIC uses IDNA2008
            }

            break;
            
          case "-ai":
            if (authInfo != null)
            {
              printRepeatedOptionError (pw, option);
              success = false;

            } else
            {
              authInfo = args.nextValue ();

              if (authInfo == null)
              {
                pw.println (option + ": authinfo expected");
                success = false;
              }
            }

            break;
            
          case "-d":
            if (duration != null || date != null)
            {
              printRepeatedOptionError (pw, option);
              success = false;
              
            } else
            {
              Integer days = parseSingleIntValueOption (args, pw, option, 
                "duration");
              
              if (days != null)
                duration = Duration.ofDays (days);
               else
                success = false;
            }
            break;
            
          case "-t":
            if (duration != null || date != null)
            {
              printRepeatedOptionError (pw, option);
              success = false;
              
            } else
            {
              String temp = parseSingleValueOption (args, pw, option, "date");
              
              if (temp != null)
              {
                try
                {
                  date = Instant.parse (temp);
                  
                } catch (DateTimeParseException e)
                {
                  pw.println (option + ": invalid date \"" + temp + "\"");
                  success = false;
                }
              } else
              {
                success = false;
              }
            }
            
          default:
            pw.println (option + ": unknown option");
            success = false;
        }
      }
      
      if (uName == null)
      {
        pw.println ("domain name must be specified");
        success = false;
      }
      
      if (authInfo == null)
      {
        pw.println ("authinfo must be specified");
        success = false;
      }
      
      if (!success)
        return null;
      
      AuthInfo1CreateRequest req = new AuthInfo1CreateRequest ();
      
      if (aName != null)
        req.setDomainName (uName, aName);
       else
        req.setDomainName (uName);
      
      req.setAuthInfo (authInfo);
      
      if (date != null)
        req.setExpirationDate (Date.from (date));
       else if (duration != null)
        req.setExpirationDate (Date.from (Instant.now ().plus (duration)));
      
      return req;
    }
    
    
    
    /**
     *  return the name of the operation
     * 
     *  @return               the name
     */
    
    @Override
    public String getName ()
    {
      return "authinfo1-create";
    }
    
    
    
    /**
     *  print the help for the operation; the output shall not exceed
     *  80 characters per line
     * 
     *  @param pw             the print writer to use
     */
    
    @Override
    public void printHelp (PrintWriter pw)
    {
      pw.println (getName () + " -dn <u-name> [ <a-name> ] -ai <authinfo> " +
        "[ -t <date> | -d <days>");
      
      pw.println ();
      
      pw.println ("  -dn <u-name>            the domain name; if a-name is " +
        "omitted, it is");
      
      pw.println ("     [ <a-name> ]         generated (due to lack of JRE " +
        "support, IDNA2003");
      
      pw.println ("                          is used, while DENIC uses " +
        "IDNA2008)");
      
      pw.println ("  -ai <authinfo>          use the given authinfo to query " +
        "domains from");
      
      pw.println ("  -t <date>               use the given date (in " +
        "ISO 8901 format) as the");
      
      pw.println ("                          expiration date of the authinfo");
      
      pw.println ("  -d <days>               let the authinfo expire in the " +
        "given number of days");
      
      pw.println ("                          other regaccs");
      pw.println ("  -r                      use recursion");
    }
    
    
    
    /**
     *  create a command from the arguments; note that in case of errors,
     *  no help shall be printed (i.e. call to {@link Operation#printHelp}
     * 
     *  @param args           the arguments
     *  @param pw             the print writer to use for error output
     *  @return               the command or {@code null} if the arguments
     *                        could not be parsed
     */
    
    @Override
    public Command parse (Arguments args, PrintWriter pw)
    {
      Request req = parseData (args, pw);
      
      if (req == null)
        return null;
      
      return req::execute;
    }
  }
    
  

  

  
  /**
   *  operation to retrieve the oldest message
   * 
   *  @version  $Revision: 1000 $
   *  @author   Klaus Malorny
   */
  
  private static class QueueRead extends BaseOperation
  {
    
    /**
     *  parse the arguments
     * 
     *  @param args           the arguments
     *  @param pw             the print writer to use for error output
     *  @return               the request or {@code null} if the arguments
     *                        could not be parsed
     */
    
    protected Request parseData (Arguments args, PrintWriter pw)
    {
      Message.Type type = null;
      boolean success = true;
      
      while (args.hasMore ())
      {
        if (!args.isOption ())
        {
          pw.println ("option expected");
          return null;
        }
        
        String option = args.nextArg ();
        
        switch (option)
        {
          case "-t":
            if (type != null)
            {
              printRepeatedOptionError (pw, option);
              success = false;
            
            } else
            {
              String temp = parseSingleValueOption (args, pw, option, "type");
              
              if (temp == null)
              {
                success = false;
                
              } else
              {
                type = Message.Type.fromToken (temp);

                if (type == null)
                {
                  pw.println (option + ": unknown type \"" + temp + "\"");
                  success = false;
                  break;
                }
              }
            }

            break;
            
          default:
            pw.println (option + ": unknown option");
            success = false;
        }
      }

      if (!success)
        return null;
      
      return type != null ? 
        new QueueReadRequest (type) : new QueueReadRequest ();
    }
    
    
    
    /**
     *  return the name of the operation
     * 
     *  @return               the name
     */
    
    @Override
    public String getName ()
    {
      return "queue-read";
    }
    
    
    
    /**
     *  print the help for the operation; the output shall not exceed
     *  80 characters per line
     * 
     *  @param pw             the print writer to use
     */
    
    @Override
    public void printHelp (PrintWriter pw)
    {
      pw.println (getName () + " [ -t <type> ]");
      
      pw.println ();
      
      pw.println ("  -t <type>               the desired message type, " +
        "may be one of:");

      for (Message.Type type: Message.Type.values ())
        pw.println ("                            " + type.getDenicToken ());
    }
    
    
    
    /**
     *  create a command from the arguments; note that in case of errors,
     *  no help shall be printed (i.e. call to {@link Operation#printHelp}
     * 
     *  @param args           the arguments
     *  @param pw             the print writer to use for error output
     *  @return               the command or {@code null} if the arguments
     *                        could not be parsed
     */
    
    @Override
    public Command parse (Arguments args, PrintWriter pw)
    {
      Request req = parseData (args, pw);
      
      if (req == null)
        return null;
      
      return req::execute;
    }
  }
    
  
  

  
  /**
   *  operation to delete a message
   * 
   *  @version  $Revision: 1000 $
   *  @author   Klaus Malorny
   */
  
  private static class QueueDelete extends BaseOperation
  {
    
    /**
     *  parse the arguments
     * 
     *  @param args           the arguments
     *  @param pw             the print writer to use for error output
     *  @return               the request or {@code null} if the arguments
     *                        could not be parsed
     */
    
    protected Request parseData (Arguments args, PrintWriter pw)
    {
      String id = null;
      Message.Type type = null;
      boolean success = true;
      
      while (args.hasMore ())
      {
        if (!args.isOption ())
        {
          pw.println ("option expected");
          return null;
        }
        
        String option = args.nextArg ();
        
        switch (option)
        {
          case "-t":
            if (type != null)
            {
              printRepeatedOptionError (pw, option);
              success = false;
            
            } else
            {
              String temp = parseSingleValueOption (args, pw, option, "type");
              
              if (temp == null)
              {
                success = false;
                
              } else
              {
                type = Message.Type.fromToken (temp);

                if (type == null)
                {
                  pw.println (option + ": unknown type \"" + temp + "\"");
                  success = false;
                  break;
                }
              }
            }

            break;
            
          case "-id":
            if (id != null)
            {
              printRepeatedOptionError (pw, option);
              success = false;
            
            } else
            {
              id = parseSingleValueOption (args, pw, option, "ID");
              
              if (id == null)
                success = false;
            }

            break;
            
          default:
            pw.println (option + ": unknown option");
            success = false;
        }
      }
      
      if (id == null)
      {
        pw.println ("ID must be specified");
        success = false;
      }

      if (!success)
        return null;
      
      return type != null ? 
        new QueueDeleteRequest (id, type) : new QueueDeleteRequest (id);
    }
    
    
    
    /**
     *  return the name of the operation
     * 
     *  @return               the name
     */
    
    @Override
    public String getName ()
    {
      return "queue-delete";
    }
    
    
    
    /**
     *  print the help for the operation; the output shall not exceed
     *  80 characters per line
     * 
     *  @param pw             the print writer to use
     */
    
    @Override
    public void printHelp (PrintWriter pw)
    {
      pw.println (getName () + " -id <id> [ -t <type> ]");
      
      pw.println ();
      
      pw.println ("  -id <id>                the ID of the message to delete");
      pw.println ("  -t <type>               the desired message type, " +
        "may be one of:");

      for (Message.Type type: Message.Type.values ())
        pw.println ("                            " + type.getDenicToken ());
    }
    
    
    
    /**
     *  create a command from the arguments; note that in case of errors,
     *  no help shall be printed (i.e. call to {@link Operation#printHelp}
     * 
     *  @param args           the arguments
     *  @param pw             the print writer to use for error output
     *  @return               the command or {@code null} if the arguments
     *                        could not be parsed
     */
    
    @Override
    public Command parse (Arguments args, PrintWriter pw)
    {
      Request req = parseData (args, pw);
      
      if (req == null)
        return null;
      
      return req::execute;
    }
  }
    
  
  
  

  
  /**
   *  operation to delete all messages
   * 
   *  @version  $Revision: 1000 $
   *  @author   Klaus Malorny
   */
  
  private static class QueueDeleteAll extends BaseOperation
  {
    
    /**
     *  parse the arguments
     * 
     *  @param args           the arguments
     *  @param pw             the print writer to use for error output
     *  @return               the type or {@code null} if the arguments
     *                        could not be parsed
     */
    
    protected Optional<Message.Type> parseData (Arguments args, PrintWriter pw)
    {
      Message.Type type = null;
      boolean success = true;
      
      while (args.hasMore ())
      {
        if (!args.isOption ())
        {
          pw.println ("option expected");
          return null;
        }
        
        String option = args.nextArg ();
        
        switch (option)
        {
          case "-t":
            if (type != null)
            {
              printRepeatedOptionError (pw, option);
              success = false;
            
            } else
            {
              String temp = parseSingleValueOption (args, pw, option, "type");
              
              if (temp == null)
              {
                success = false;
                
              } else
              {
                type = Message.Type.fromToken (temp);

                if (type == null)
                {
                  pw.println (option + ": unknown type \"" + temp + "\"");
                  success = false;
                  break;
                }
              }
            }

            break;
            
          default:
            pw.println (option + ": unknown option");
            success = false;
        }
      }
      
      if (!success)
        return null;
      
      return Optional.ofNullable (type);
    }
    
    
    
    /**
     *  return the name of the operation
     * 
     *  @return               the name
     */
    
    @Override
    public String getName ()
    {
      return "queue-delete-all";
    }
    
    
    
    /**
     *  print the help for the operation; the output shall not exceed
     *  80 characters per line
     * 
     *  @param pw             the print writer to use
     */
    
    @Override
    public void printHelp (PrintWriter pw)
    {
      pw.println (getName () + " [ -t <type> ]");
      
      pw.println ();
      
      pw.println ("  -t <type>               the desired message type, " +
        "may be one of:");

      for (Message.Type type: Message.Type.values ())
        pw.println ("                            " + type.getDenicToken ());
    }
    
    
    
    /**
     *  create a command from the arguments; note that in case of errors,
     *  no help shall be printed (i.e. call to {@link Operation#printHelp}
     * 
     *  @param args           the arguments
     *  @param pw             the print writer to use for error output
     *  @return               the command or {@code null} if the arguments
     *                        could not be parsed
     */
    
    @Override
    public Command parse (Arguments args, PrintWriter pw)
    {
      Optional<Message.Type> type = parseData (args, pw);
      
      if (type == null)
        return null;
      
      return port ->
        {
          while (true)
          {
            QueueReadRequest req = type.isPresent () ?
              new QueueReadRequest (type.get ()) : new QueueReadRequest ();
            
            QueueReadResponse resp = req.execute (port);

            if (resp.isSuccess ())
            {
              Message msg = resp.getMessage ();
              if (msg == null) break;

              Request req2 = 
                new QueueDeleteRequest (msg.getMessageID (), msg.getType ());

              Response resp2 = req2.execute (port);

              if (!resp2.isSuccess ())
              {
                System.err.println ("delete failed:\n" + resp2);
                break;
              }

            } else
            {
              System.err.println ("queue read failed:\n" + resp);
              break;
            }
          }
        };
    }
  }
    
  

  
}

